part of masamune;

extension WidgetRefTextEditingControllerExtensions on WidgetRef {
  TextEditingController useTextEditingController(
    String key, [
    String defaultValue = "",
  ]) {
    return containerBuilder<_TextEditingControllerContainer,
        TextEditingController>(
      key: "textEditingController:$key",
      containerBuilder: (container) {
        if (container?.controller != null) {
          final controller = container!.controller;
          controller.text = defaultValue;
          return _TextEditingControllerContainer(
            controller: controller,
            initialValue: defaultValue,
          );
        }
        final controller = TextEditingController(text: defaultValue);
        return _TextEditingControllerContainer(
          controller: controller,
          initialValue: defaultValue,
        );
      },
      resultBuilder: (container) {
        return container.controller;
      },
      test: (container) => container.initialValue != defaultValue,
    );
  }

  Map<String, TextEditingController> useTextEditingControllerMap(
    String key,
    Map defaultValues,
  ) {
    return containerBuilder<_TextEditingControllerMapContainer,
        Map<String, TextEditingController>>(
      key: "textEditingControllerMap:$key",
      containerBuilder: (container) {
        if (container?.controllers != null &&
            container!.controllers.isNotEmpty) {
          final controllers =
              Map<String, TextEditingController>.from(container.controllers);
          for (final tmp in defaultValues.entries) {
            if (controllers.containsKey(tmp.key)) {
              controllers[tmp.key.toString()] = tmp.value;
            } else {
              controllers[tmp.key.toString()] =
                  TextEditingController(text: tmp.value.toString());
            }
          }
          controllers.removeWhere((key, value) {
            return !defaultValues.containsKey(key);
          });

          return _TextEditingControllerMapContainer(
            controllers: controllers,
            initialValues: defaultValues,
          );
        }

        final controllers = defaultValues.map<String, TextEditingController>(
          (key, value) => MapEntry(
            key.toString(),
            TextEditingController(text: value.toString()),
          ),
        );
        return _TextEditingControllerMapContainer(
          controllers: controllers,
          initialValues: defaultValues,
        );
      },
      resultBuilder: (container) {
        return container.controllers;
      },
      test: (container) => container.initialValues != defaultValues,
    );
  }
}

@immutable
class _TextEditingControllerContainer extends VariableContainer {
  _TextEditingControllerContainer({
    required this.controller,
    required this.initialValue,
  });

  final TextEditingController controller;
  final String initialValue;

  @override
  void didUpdateContainer(_TextEditingControllerContainer oldContainer) {
    super.didUpdateContainer(oldContainer);
    if (controller != oldContainer.controller) {
      oldContainer.dispose();
    }
  }

  @override
  void dispose() {
    super.dispose();
    controller.dispose();
  }
}

@immutable
class _TextEditingControllerMapContainer extends VariableContainer {
  _TextEditingControllerMapContainer({
    required this.controllers,
    required this.initialValues,
  });

  final Map<String, TextEditingController> controllers;
  final Map initialValues;

  @override
  void didUpdateContainer(_TextEditingControllerMapContainer oldContainer) {
    super.didUpdateContainer(oldContainer);
    if (controllers != oldContainer.controllers) {
      oldContainer.dispose();
    }
  }

  @override
  void dispose() {
    super.dispose();
    controllers.values.forEach((e) => e.dispose());
  }
}

class _TextEditingControllerMapHookCreator {
  const _TextEditingControllerMapHookCreator();

  /// Creates a map of [String] and [TextEditingController] that will be disposed automatically.
  ///
  /// The [text] parameter can be used to set the initial value of the
  /// controller.
  Map<String, TextEditingController> call(
      [Map<String, String>? map, List<Object?>? keys]) {
    return use(_TextEditingControllerMapHook(map, keys));
  }

  /// Creates a map of [String] and [TextEditingController] from the initial [value] that will
  /// be disposed automatically.
  Map<String, TextEditingController> fromValue(
    Map<String, TextEditingValue>? map, [
    List<Object?>? keys,
  ]) {
    return use(_TextEditingControllerMapHook.fromValue(map, keys));
  }
}

/// Creates a map of [String] and [TextEditingController], either via an initial text or an initial
/// [TextEditingValue].
///
/// To use a [TextEditingController] with an optional initial text, use
/// ```dart
/// final controller = useTextEditingController(text: 'initial text');
/// ```
///
/// To use a [TextEditingController] with an optional initial value, use
/// ```dart
/// final controller = useTextEditingController
///   .fromValue(TextEditingValue.empty);
/// ```
///
/// Changing the text or initial value after the widget has been built has no
/// effect whatsoever. To update the value in a callback, for instance after a
/// button was pressed, use the [TextEditingController.text] or
/// [TextEditingController.text] setters. To have the [TextEditingController]
/// reflect changing values, you can use [useEffect]. This example will update
/// the [TextEditingController.text] whenever a provided [ValueListenable]
/// changes:
/// ```dart
/// final controller = useTextEditingController();
/// final update = useValueListenable(myTextControllerUpdates);
///
/// useEffect(() {
///   controller.text = update;
/// }, [update]);
/// ```
///
/// See also:
/// - [TextEditingController], which this hook creates.
const useTextEditingControllerMap = _TextEditingControllerMapHookCreator();

class _TextEditingControllerMapHook
    extends Hook<Map<String, TextEditingController>> {
  const _TextEditingControllerMapHook(
    this.initialTextMap, [
    List<Object?>? keys,
  ])  : initialValueMap = null,
        super(keys: keys);

  const _TextEditingControllerMapHook.fromValue(
    this.initialValueMap, [
    List<Object?>? keys,
  ])  : initialTextMap = null,
        super(keys: keys);

  final Map<String, String>? initialTextMap;
  final Map<String, TextEditingValue>? initialValueMap;

  @override
  _TextEditingControllerMapHookState createState() {
    return _TextEditingControllerMapHookState();
  }
}

class _TextEditingControllerMapHookState extends HookState<
    Map<String, TextEditingController>, _TextEditingControllerMapHook> {
  late final Map<String, TextEditingController> _map;
  @override
  void initHook() {
    _map = hook.initialTextMap?.map(
          (k, v) => MapEntry(k, TextEditingController(text: v)),
        ) ??
        hook.initialValueMap?.map(
          (k, v) => MapEntry(k, TextEditingController.fromValue(v)),
        ) ??
        <String, TextEditingController>{};
  }

  @override
  Map<String, TextEditingController> build(BuildContext context) => _map;

  @override
  void dispose() {
    _map.forEach((key, value) => value.dispose());
    _map.clear();
  }

  @override
  String get debugLabel => 'useTextEditingControllerMap';
}
